package de.fhmuenster.masterthesis.generator;

public class TestUtilsGenerator extends AbstractTestgenerator {

	private static final String CLASS_DECLARATION = "public class TestUtils {";

	private StringBuilder codegeneratorUtils = new StringBuilder();

	private String packageName;

	public TestUtilsGenerator(String packageName) {
		this.packageName = packageName;
	}

	public void generateUtils() {
		addPackageDeclaration(packageName, codegeneratorUtils);
		addSeparatorLine(codegeneratorUtils);

		addImports();
		addSeparatorLine(codegeneratorUtils);

		addUtilsClassHeader();
		addSeparatorLine(codegeneratorUtils);

		addUtilMethods();

		addClassClosing(codegeneratorUtils);
	}

	private void addImports() {
		appendLine("import static org.camunda.bpm.engine.test.assertions.ProcessEngineTests.*;", codegeneratorUtils);
		appendLine("import java.util.List;", codegeneratorUtils);
		appendLine("import java.util.Optional;", codegeneratorUtils);
		appendLine("import org.camunda.bpm.engine.externaltask.LockedExternalTask;", codegeneratorUtils);
		appendLine("import org.camunda.bpm.engine.test.assertions.bpmn.BpmnAwareTests;", codegeneratorUtils);
	}

	private void addUtilsClassHeader() {
		appendLine(CLASS_DECLARATION, codegeneratorUtils);
	}

	private void addUtilMethods() {
		appendLine(
				"\tpublic static LockedExternalTask getLockedTask(List<LockedExternalTask> lockedTasks, String activityId) {",
				codegeneratorUtils);
		appendLine("\t\tOptional<LockedExternalTask> findOpt = lockedTasks.stream() //", codegeneratorUtils);
		appendLine("\t\t\t.filter(locked -> locked.getActivityId().equals(activityId)) //", codegeneratorUtils);
		appendLine("\t\t\t.findFirst(); //", codegeneratorUtils);

		addSeparatorLine(codegeneratorUtils);

		appendLine("\t\treturn findOpt.isPresent() ? findOpt.get() : null;", codegeneratorUtils);
		appendLine("\t}", codegeneratorUtils);

		addSeparatorLine(codegeneratorUtils);

		appendLine("\tpublic static List<LockedExternalTask> lockTasks(String topic) {", codegeneratorUtils);
		appendLine("\t\treturn fetchAndLock(topic, BpmnAwareTests.DEFAULT_WORKER_EXTERNAL_TASK, 10);",
				codegeneratorUtils);
		appendLine("\t}", codegeneratorUtils);

		addSeparatorLine(codegeneratorUtils);

		appendLine("\tpublic static List<LockedExternalTask> lockTasks(List<LockedExternalTask> alreadyLockedTasks, String topic) {",
				codegeneratorUtils);
		appendLine("\t\tList<LockedExternalTask> lockedTasks = lockTasks(topic);", codegeneratorUtils);
		appendLine("\t\tlockedTasks.addAll(alreadyLockedTasks);", codegeneratorUtils);
		
		addSeparatorLine(codegeneratorUtils);

		appendLine("\t\treturn lockedTasks;", codegeneratorUtils);
		appendLine("\t}", codegeneratorUtils);
	}

	public String getUtilsCode() {
		return codegeneratorUtils.toString();
	}
}
