package de.fhmuenster.masterthesis.utils;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import org.eclipse.emf.common.util.EList;

import de.fhmuenster.masterthesis.testgeneratorDSL.Compare;
import de.fhmuenster.masterthesis.testgeneratorDSL.DelegateMock;
import de.fhmuenster.masterthesis.testgeneratorDSL.EndCheck;
import de.fhmuenster.masterthesis.testgeneratorDSL.EndChecks;
import de.fhmuenster.masterthesis.testgeneratorDSL.ExternalServiceMock;
import de.fhmuenster.masterthesis.testgeneratorDSL.Flow;
import de.fhmuenster.masterthesis.testgeneratorDSL.FlowElement;
import de.fhmuenster.masterthesis.testgeneratorDSL.FlowElementReference;
import de.fhmuenster.masterthesis.testgeneratorDSL.Loop;
import de.fhmuenster.masterthesis.testgeneratorDSL.ManualMock;
import de.fhmuenster.masterthesis.testgeneratorDSL.Mock;
import de.fhmuenster.masterthesis.testgeneratorDSL.MockingRule;
import de.fhmuenster.masterthesis.testgeneratorDSL.StartFlowElement;
import de.fhmuenster.masterthesis.testgeneratorDSL.Test;
import de.fhmuenster.masterthesis.testgeneratorDSL.UserTaskFlowElement;
import de.fhmuenster.masterthesis.testgeneratorDSL.VariableDeclaration;
import de.fhmuenster.masterthesis.testgeneratorDSL.VariableDeclarations;

public class TestgeneratorDSLUtils {

	public static List<de.fhmuenster.masterthesis.testgeneratorDSL.Boolean> mapBoolean(List<Boolean> bools) {
		return bools.stream() //
				.map(bool -> mapBoolean(bool)) //
				.collect(Collectors.toList()); //
	}

	public static List<Boolean> mapDSLBoolean(List<de.fhmuenster.masterthesis.testgeneratorDSL.Boolean> bools) {
		return bools.stream() //
				.map(bool -> mapDSLBoolean(bool)) //
				.collect(Collectors.toList()); //
	}

	public static de.fhmuenster.masterthesis.testgeneratorDSL.Boolean mapBoolean(Boolean bool) {
		if (bool == null) {
			return null;
		}

		if (bool) {
			return de.fhmuenster.masterthesis.testgeneratorDSL.Boolean.TRUE;
		}

		return de.fhmuenster.masterthesis.testgeneratorDSL.Boolean.FALSE;
	}

	public static Boolean mapDSLBoolean(de.fhmuenster.masterthesis.testgeneratorDSL.Boolean bool) {
		if (bool == null) {
			return null;
		}

		if (bool.equals(de.fhmuenster.masterthesis.testgeneratorDSL.Boolean.TRUE)) {
			return Boolean.TRUE;
		}

		return Boolean.FALSE;
	}

	public static List<MockingRule> getMockingRules(ExternalServiceMock mock) {
		return toList(mock.getMockingRules());
	}
	
	public static List<Mock> getMocks(Test testcase) {
		return testcase.getMocks().stream() //
			.map(mockRef -> mockRef.getRef()) //
			.collect(Collectors.toList());
	}
	
	public static List<ManualMock> getManualMocks(DelegateMock mock) {
		return toList(mock.getManualMocks());
	}
	
	public static List<VariableDeclaration> getVariableDeclarations(MockingRule mockingRule) {
		return toList(mockingRule.getVariables());
	}
	
	public static List<Compare> getComparisons(MockingRule mockingRule) {
		return toList(mockingRule.getComparisons());
	}
	
	public static List<VariableDeclarations> getVariableDeclarations(Test test) {
		return toList(test.getDeclarations());
	}
	
	public static List<VariableDeclaration> getVariables(VariableDeclarations declarations) {
		return toList(declarations.getVariables());
	}

	public static List<FlowElementReference> getInclElements(Flow flow) {
		return toList(flow.getInclElements());
	}

	public static List<FlowElement> getUserInputElements(Flow flow) {
		return flow.getInclElements().stream() //
				.map(incl -> incl.getRef()) //
				.filter(incl -> isUserInputElement(incl)) //
				.collect(Collectors.toList()); //
	}
	
	public static <T extends Object> List<T> toList(EList<T> list) {
		List<T> dest = new ArrayList<>();
		list.forEach(obj -> dest.add(obj));

		return dest;
	}

	public static List<EndCheck> getEndChecks(Test test) {
		EndChecks endCheck = test.getEndCheck();
		if(endCheck != null) {
			return toList(endCheck.getEndChecks());
		} else {
			return new ArrayList<>();
		}
	}

	private static boolean isUserInputElement(FlowElement flowElement) {
		return flowElement instanceof UserTaskFlowElement || //
				flowElement instanceof StartFlowElement; //
	}

	public static List<Integer> getFrequencies(Loop dslLoop) {
		return toList(dslLoop.getFrequencies());
	}
}
